// file      : xsd/cxx/tree/tree-forward.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx/tree/tree-forward.hxx>

#include <xsd-frontend/semantic-graph.hxx>
#include <xsd-frontend/traversal.hxx>


namespace CXX
{
  namespace Tree
  {
    namespace
    {
      struct List : Traversal::List,
                    protected virtual Context
      {
        List (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& l)
        {
          os << "class " << ename (l) << ";";
        }
      };

      struct Union : Traversal::Union,
                     protected virtual Context
      {
        Union (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& u)
        {
          os << "class " << ename (u) << ";";
        }
      };

      struct Complex : Traversal::Complex,
                       protected virtual Context
      {
        Complex (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& c)
        {
          os << "class " << ename (c) << ";";
        }
      };
    }

    Void
    generate_forward (Context& ctx, SemanticGraph::Schema& tu)
    {
      ctx.os << "// Forward declarations." << endl
             << "//" << endl;

      Traversal::Schema schema;
      Traversal::Sources sources;
      Traversal::Names names_ns, names;
      Namespace ns (ctx, false);

      List list (ctx);
      Union union_ (ctx);
      Complex complex (ctx); // Also handles Enumeration.

      schema >> sources >> schema;
      schema >> names_ns >> ns >> names;

      names >> list;
      names >> union_;
      names >> complex;

      schema.dispatch (tu);

      ctx.os << endl;
    }
  }
}
