// file      : xsd/cxx/tree/tree-template.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx/tree/tree-template.hxx>

#include <xsd-frontend/semantic-graph.hxx>
#include <xsd-frontend/traversal.hxx>

#include <backend-elements/regex.hxx>

namespace CXX
{
  namespace Tree
  {
    namespace
    {
      struct Includes : Traversal::Imports,
                        Traversal::Includes,
                        protected virtual Context
      {
        Includes (Context& c, String const& expr)
            : Context (c), expr_ (expr)
        {
        }

        virtual Void
        traverse (SemanticGraph::Imports& i)
        {
          traverse (i.path ());
        }

        virtual Void
        traverse (SemanticGraph::Includes& i)
        {
          traverse (i.path ());
        }

        virtual Void
        traverse (SemanticGraph::Path const& path)
        {
          using BackendElements::Regex::perl_s;

          // Note the use of the portable representation of the path.
          //
          String name (perl_s (String (path.string ()), expr_));

          Char op (include_with_brackets ? '<' : '"');
          Char cl (include_with_brackets ? '>' : '"');

          os << "#include " << op << include_prefix << name << cl << endl
             << endl;
        }

      private:
        String expr_;
      };
    }

    Void
    generate_tree_template (Context& ctx,
                            SemanticGraph::Schema& tu,
                            String const& expr)
    {
      // Generate includes.
      //
      {
        Traversal::Schema schema;
        Includes includes (ctx, expr);

        schema >> includes;

        schema.dispatch (tu);
      }
    }
  }
}
